# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
InVirtualEnv Python Virtualenv Installer Module
"""
import json
import os
import sys


__copyright__ = "Copyright 2016, Yahoo Inc."
__directory__ = os.path.dirname(__file__)
__version__ = '0.0.0'

if hasattr(sys, "_MEIPASS"):
    __directory__ = sys._MEIPASS  # pragma: no cover

__metadata_filename__ = os.path.join(__directory__, 'package_metadata.json')
if os.path.exists(__metadata_filename__):  # pragma: no cover
    with open(__metadata_filename__) as __metadata_handle:
        __metadata__ = json.load(__metadata_handle)
        __version__ = __metadata__['version']

del __metadata_filename__

__all__ = [
    'config',
    'deploy',
    'exceptions',
    'package',
    'plugin',
    'utility',
    'virtualenv'
]



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
Functions for managing packaging
"""
import os
import platform
from xml.etree import ElementTree
import subprocess

try:  # pragma: no cover
    import configparser as ConfigParser
except ImportError:  # pragma: no cover
    import ConfigParser

import requests

from .exceptions import BuildException
from .utility import display_header
from distutils.version import LooseVersion


def package_scripts_directory():
    """
    Get the package scripts directory

    Returns
    str
        The path to the directory containing the package scripts
    """
    module_directory = os.path.dirname(__file__)
    script_directory = os.path.join(
        module_directory,
        'package_scripts'
    )
    return script_directory


def strip_from_end(text, suffix):
    """
    Strip a substring from the end of a string

    Parameters
    ----------
    text : str
        The string to be evaluated

    suffix : str
        The suffix or substring to remove from the end of the text string

    Returns
    -------
    str
        A string with the substring removed if it was found at the end of the
        string.
    """
    if not text.endswith(suffix):
        return text
    return text[:len(text)-len(suffix)]


def package_versions(package, pypi_url=None):
    """
    Get all versions of a package from pypi

    Parameters
    ----------
    package : str
        The python package to search for

    pypi_url : str, optional
        The pypi URL to send the request to, defaults to the production
        pypi endpoint.

    Returns
    -------
    list
        A list of all packages found on pypi.  The list will be
        empty if there were no versions found.
    """
    if pypi_url:  # pragma: no cover
        url = pypi_url
    else:
        index_url = 'https://pypi.python.org/simple'
        if os.path.exists('/etc/pip.conf'):  # pragma: no cover
            cfg = ConfigParser.SafeConfigParser({'index-url': index_url})
            cfg.read('/etc/pip.conf')
            index_url = cfg.get('global', 'index-url')
        url = '%s/%s/' % (index_url, package)
    versions = []
    response = requests.get(url)
    if response.status_code in [404]:  # pragma: no cover
        return []
    result = response.text
    tree = ElementTree.fromstring(result)
    for element in tree.findall('.//a/[@rel="internal"]'):
        possible_package = element.text
        if possible_package.endswith('.tar.gz'):
            possible_package = strip_from_end(possible_package, '.tar.gz')
            possible_package = possible_package.split('-')
            version = possible_package[-1]
            versions.append(version)

    versions.sort(key=LooseVersion)
    return versions


def latest_package_version(package):
    """
    Get the latest version number for a package

    Parameters
    ----------
    package : str
        Package to get the latest version of

    Returns
    -------
    str
        Latest package version or an empty string if no versions are found
    """
    versions = package_versions(package)
    if not versions:
        return ''
    return versions[-1]


def install_prereq_packages(test=False):  # pragma: no cover
    """
    Install packages required to build python

    Parameters
    ----------
    test: bool, optional
        Don't run the actual command, if True
    """
    resulting_files = [
        '/usr/bin/gcc',
        '/usr/bin/make'
    ]
    needed_packages = [
        'gcc',
        'make',
        'git',
        'libffi',
        'libffi-devel',
        'gdb',
        'glibc-devel',
        'libstdc++-devel',
        'libcom_err-devel',
        'krb5-devel',
        'openssl-devel',
        'gettext-devel',
        'gdbm-devel',
        'zlib-devel',
        'libsepol-devel',
        'sqlite-devel',
        'tk-devel',
        'expat-devel',
        'readline-devel',
        'ncurses-devel',
        'bzip2-devel',
        'xz-devel',
        'gmp-devel',
        'mysql-devel',

        # Packages not in RHEL releases before RHEL6
        'db4-devel',
        'libpcap-devel',
    ]
    if platform.system() in ['Darwin']:
        print('No prereq packages defined for platform %s' % platform.system())
        return
    redhat_release = '7.0'
    if os.path.exists('/etc/redhat-release'):
        redhat_release = open('/etc/redhat-release').readlines()[0].strip().split()[-2]
    display_header('Installing build system needed for build on Redhat %s' % redhat_release)
    display_header('Installing additional build dependencies')
    if not test:  # pragma: no cover
        subprocess.check_output(['yum', '-y', 'install'] + needed_packages)

    display_header('Verifying needed dependencies where installed')
    for filename in resulting_files:
        if not test and not os.path.exists(filename):  # pragma: no cover
            raise BuildException('Required file %s is missing' % filename)

    if redhat_release.startswith('6'):
        display_header('Fixing the user .gitconfig to work with git < 1.8')
        gitconfig = os.path.exists(os.path.expanduser('~/.gitconfig'))
        if os.path.exists(gitconfig):
            old_git_config = []
            with open(gitconfig) as file_handle:
                old_git_config = file_handle.read().split('\n')

            with open(gitconfig, 'w') as file_handle:
                for line in old_git_config:
                    if 'default=simple' not in line.replace(' ', ''):
                        file_handle.write(line + '\n')



#!/usr/bin/env python3
from __future__ import print_function
import logging
import os
import shutil
import sys
from invirtualenv.deploy import unlink_deployed_bin_files
from invirtualenv.config import get_configuration_dict


if __name__ == "__main__":
    if not os.path.exists('deploy.conf'):
        print("No 'deploy.conf' found.  Doing nothing", file=sys.stderr)
        sys.exit(0)

    config = get_configuration_dict()
    venv_dir = config['global'].get('virtualenv_deploy_dir', None)

    if venv_dir and os.path.exists(venv_dir):
        unlink_deployed_bin_files(venv_dir)
        logging.debug('Removing virtualenv directory %r' % venv_dir)
        shutil.rmtree(venv_dir)



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

"""
General utility functionality module
"""
from __future__ import print_function
import logging
import os
import textwrap
import sys
from jinja2 import Template
from distutils.spawn import find_executable
from .exceptions import CommandNotFound


logger = logging.getLogger(__name__)  # pylint: disable=C0103


def get_terminal_size():
    """
    Get the terminal rows and columns if we are running on an
    interactive terminal.
    Returns
    -------
    rows : int
        The number of rows on the current terminal.
    columns : int
        The number of columns on the current terminal.
    """
    try:
        rows, columns = os.popen('stty size', 'r').read().split()
    except ValueError:  # pragma: no cover
        rows = 24
        columns = 80
    return int(rows), int(columns)


def display_header(
        text='', width=None, separator=None, outfile=None, collapse=False
):
    """
    Display a textual header message.
    Parameters
    ----------
    text : str
        The text to print/display
    width : int, optional
        The width (text wrap) of the header message.
        This will be the current terminal width as determined
        by the 'stty size' shell command if not specified.
    separator : str, optional
        The character or string to use as a horizontal
        separator.  Will use '=' if one is not specified.
    outfile : File, optional
        The File object to print the header text to.
        This will use sys.stdout if not specified.
    :return:
    """
    if not outfile:
        outfile = sys.stdout
    width = width or get_terminal_size()[1]
    separator = separator or '='
    # Screwdriver buffers stdout and stderr separately.  This can
    # cause output from previous operations to show after our
    # header text.  So we flush the output streams to ensure
    # all existing output is sent/displayed before printing our
    # header.
    sys.stderr.flush()
    outfile.flush()

    if collapse:
        for line in textwrap.wrap(text, width=width-4):
            header_line = separator + ' ' + line + ' ' + \
                separator * (width - 4 - len(line))
            print(header_line, file=outfile)
    else:
        # Print the header text
        horiz = separator * width
        print(horiz, file=outfile)
        print(
            os.linesep.join(textwrap.wrap(text, width=width)),
            file=outfile
        )
        print(horiz, file=outfile)

    # Once again flush our header to the output stream so things
    # show up in the correct order.
    sys.stderr.flush()
    outfile.flush()


def which(command):
    """
    Function searches the path for the command executable

    Parameters
    ----------
    command : str

    Returns
    -------
    str
        Path to the command

    Raises
    ------
    CommandNotFound:
        Command was not found
    """
    bin_dir = os.path.dirname(sys.executable)
    bin_exe = os.path.join(bin_dir, command)
    if os.path.exists(bin_exe) and os.access(bin_exe, os.X_OK):  # pragma: no cover
        return bin_exe
    result = find_executable(command)
    if not result:  # pragma: no cover
        raise CommandNotFound('Command %r was not found in the path' % command)
    return result


def csv_list(value):
    """
    Convert a comma separated string into a list

    Parameters
    ----------
    value : str
        The string object to convert to a list

    Returns
    -------
    list
        A list based on splitting the string on the ',' character
    """
    if value:
        result = []
        for item in value.split(','):
            item = item.strip()
            if item:
                result.append(item)
        return result
    return []


def str_to_bool(value):
    """
    Convert a string too a bool object

    Parameters
    ----------
    value : str
        The string object to convert to a bool.  The following case insensitive
        strings evaluate to True ['true', 1', 'up', 'on']

    Returns
    -------
    bool
        Boolean based on the string
    """
    value = value.lower()
    return value in ['true', '1', 'up', 'on']


def str_to_list(value):
    """
    Convert a newline terminated string into a list.  Any empty lines
    will be removed from the result list.

    Parameters
    ----------
    value : str
        The string object to convert to a list

    Returns
    -------
    list
        List based on the string
    """
    result_list = []
    for item in value.strip().split('\n'):
        item = item.strip()
        if item:
            result_list.append(item)

    return result_list


def str_to_dict(value):
    """
    Convert a newline terminated string of key=value into a dictionary.

    Parameters
    ----------
    value : str
        The string object to convert to a dictionary

    Returns
    -------
    dict
        Dictionary based on the string
    """

    result_dict = {}
    for item in str_to_list(value):
        split_setting = item.split('=')
        result_dict[split_setting[0]] = '='.join(split_setting[1:])
    return result_dict


def str_format_env(value):
    """
    Substitute values with environment variables in a string

    Parameters
    ----------
    value : str
        The string object to be formatted and converted into a list

    Returns
    -------
    str
        With env variable values substituded
    """
    template = Template(value)
    result = template.render(**os.environ)
    if result != value:
        logger.debug('Rendered: %r to %r', value, result)
    return result


def change_uid_gid(user_uid=None, user_gid=None):  # pragma: no cover
    """
    preexec_fn to change the uid/gid when using subprocess

    Parameters
    ----------
    user_uid : int, optional
        The user UID to set to, does not set user uid if this is not passed

    user_gid : int, optional
        The user GID to set to, does not set the gid if this is not passed
    """
    if user_uid is not None:
        os.setuid(user_uid)
    if user_gid is not None:
        os.setgid(user_gid)


def chown_recursive(path, uid, gid):  # pragma: no cover
    """
    Change the ownership of all files and directories in path

    Parameters
    ----------
    path : str
        The root path to start changing the ownership at

    uid, : int
        The uid to change the ownership to

    gid, : int
        The gid to change the ownership to
    """
    for root, dirs, files in os.walk(path):
        for item in dirs:
            os.chown(os.path.join(root, item), uid, gid)
        for item in files:
            os.chown(os.path.join(root, item), uid, gid)


def update_recursive_generator(basedict, updatedict):
    """
    Recursively run update on a dictionary

    Parameters
    ----------
    basedict : dict
        First dictionary, which will contain the updated result

    updatedict : dict
        Second dictionary that will be be updated to basedict, conflicts will
        be replaced with the values from this dictionary.

    Returns
    -------
    dict
        basedict updated to contain the values from updatedict

    """
    for key in set(basedict.keys()) | set(updatedict.keys()):
        if key in basedict and key in updatedict:
            # Both dicts have the same key
            if isinstance(basedict[key], dict) and \
                    isinstance(updatedict[key], dict):
                # Both values are  dicts so merge them.
                yield (
                    key,
                    dict(
                        update_recursive_generator(
                            basedict[key],
                            updatedict[key]
                        )
                    )
                )
            else:
                # One of the values isn't a dict to merge so use the value from
                # updatedict.
                yield (key, updatedict[key])
        elif key in basedict:
            yield (key, basedict[key])
        else:
            yield (key, updatedict[key])


def update_recursive(basedict, updatedict):
    """
    Recursively run update on a dictionary

    Parameters
    ----------
    basedict : dict
        First dictionary, which will contain the updated result

    updatedict : dict
        Second dictionary that will be be updated to basedict, conflicts will
        be replaced with the values from this dictionary.

    Returns
    -------
    dict
        basedict updated to contain the values from updatedict

    """
    return dict(update_recursive_generator(basedict, updatedict))



#!/usr/bin/env python
# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.

import os
import json
from setuptools import setup

# Default version number
BASE_VERSION = '19.1.1'
METADATA_FILENAME = 'invirtualenv/package_metadata.json'
BASEPATH = os.path.dirname(os.path.abspath(__file__))
METADATA_FILE = os.path.join(BASEPATH, METADATA_FILENAME)


def readme():
    """
    Get the contents of the README file
    :return:
    """
    possible_filenames = ['README.rst', 'README.md', 'README.txt']
    filename = None
    data = ''
    for filename in possible_filenames:
        if os.path.exists(filename):
            break
    if filename:
        with open(filename) as file_handle:
            data = file_handle.read()
    return data


def scripts():
    """
    Get a list of the scripts in the scripts directory

    Returns
    -------
    list
        A list of strings containing the scripts in the scripts directory
    """
    script_list = []

    if os.path.isdir('scripts'):
        script_list += [
            os.path.join('scripts', f) for f in os.listdir('scripts')
        ]
    return script_list


def get_version(version_file):
    """
    Get the version number based on the BUILD_NUMBER

    Parameters
    ----------
    version_file : str
        The python file to store the version metadata in

    Returns
    -------
    str
        Version string
    """
    for env_variable in ['BUILD_NUMBER', 'TRAVIS_BUILD_NUMBER']:
        build_number = int(os.environ.get(env_variable, '0'))
        if build_number:
            break

    version = BASE_VERSION.split('.')
    version[-1] = str(build_number)
    version = '.'.join(version)
    if build_number != 0:
        with open(version_file, 'w') as version_handle:
            json.dump({'version': version}, version_handle)

    elif os.path.exists(version_file):
        with open(version_file) as version_handle:
            version = json.load(version_handle)['version']

    return version


if __name__ == '__main__':
    version = '1.0.0'
    setup(
        version=get_version(METADATA_FILENAME),
        license='LICENSE.txt',
        package_data= {
            'invirtualenv': [
                'package_metadata.json',
            ],
            'invirtualenv_plugins': [
                'docker_scripts/*'
            ]
        },
        scripts= scripts(),
    )



# Copyright (c) 2016, Yahoo Inc.
# Copyrights licensed under the BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Useful contextmanagers
"""
from __future__ import print_function
from contextlib import contextmanager
import errno
import logging
import os
import shutil
import tempfile


LOG = logging.getLogger(__name__)


@contextmanager
def working_dir(new_path):
    """
    A context manager that changes to the new_path directory and
    returns to the current working directory when it completes.
    """
    old_dir = os.getcwd()
    os.chdir(new_path)
    try:
        yield
    finally:
        os.chdir(old_dir)


@contextmanager
def revert_file(filename):
    """
    A context manager that reverts a file's contents.
    """
    original_data = None
    with open(filename, 'rb') as file_handle:
        original_data = file_handle.read()

    try:
        yield
    finally:
        if original_data:  # pragma: no cover
            with open(filename, 'wb') as file_handle:
                file_handle.write(original_data)


@contextmanager
def TemporaryDirectory():  # pylint: disable=C0103
    """
    A context manager that provides a temporary directory and cleans it up.

    Note:
        This context manager duplicates the base functionality of the
        tempfile.TemporaryDirectory() context manager in Python 3.2+

    Returns
    -------
    str
        The path to the temporary directory
    """
    name = tempfile.mkdtemp()
    try:
        yield name
    finally:
        try:
            shutil.rmtree(name)
        except OSError as error:  # pragma: no cover
            # Reraise unless ENOENT: No such file or directory
            # (ok if directory has already been deleted)
            if error.errno != errno.ENOENT:
                raise


@contextmanager
def InTemporaryDirectory():  # pylint: disable=C0103
    """
    A context manager that creates a temporary directory and changes
    the current directory into it.
    """
    with TemporaryDirectory() as tempdir:
        with working_dir(tempdir):
            yield



import os
import logging
import subprocess
import pkgutil
from invirtualenv.plugin_base import InvirtualenvPlugin
from invirtualenv.utility import find_executable


logger = logging.getLogger(__name__)


SPEC_TEMPLATE = """Summary: {{global['description']|default('No summary available')}}
Name: {{global['name']}}
Version: {{global['version']|default('0.0.0')}}
Release: {{rpm_package['release']|default('1')}}
License: {{rpm_package['license']|default('Closed Source')}}
Group: {{rpm_package['group']|default('Development')}}
{% if rpm_package['deps'] %}Requires: {% for package in rpm_package['deps'] %}{{package}}{{ ", " if not loop.last }}{% endfor %}{% endif %}
Packager: {{rpm_package['packager']|default('VerizonMedia')}}
URL: {{global['url']|default('https://github.com/yahoo/invirtualenv')}}
AutoReqProv: no
BuildArch: noarch
# If basepython is specified/set. Make it as RPM requirement.
%if {{global['basepython']}}
Requires(post): {{global['basepython']}}
%fi
Requires(post): /usr/bin/which

%description
{{rpm_package['description']|default('No description')}}

%install
mkdir -p %{buildroot}/usr/share/%{name}-%{version}/
mkdir -p %{buildroot}/usr/share/%{name}-%{version}/package_scripts/
cp -r {{rpm_package['cwd']}}/wheels %{buildroot}/usr/share/%{name}-%{version}
cp {{rpm_package['cwd']}}/deploy.conf %{buildroot}/usr/share/%{name}-%{version}/deploy.conf
cp {{rpm_package['cwd']}}/post_install.py %{buildroot}/usr/share/%{name}-%{version}/package_scripts/post_install.py
cp {{rpm_package['cwd']}}/pre_uninstall.py %{buildroot}/usr/share/%{name}-%{version}/package_scripts/pre_uninstall.py
chmod 755 %{buildroot}/usr/share/%{name}-%{version}/package_scripts/post_install.py
chmod 755 %{buildroot}/usr/share/%{name}-%{version}/package_scripts/pre_uninstall.py

%post
export PATH=$PATH:/opt/python/bin:/usr/local/bin
virtualenv_path=`which virtualenv ||:`
if [ -z $virtualenv_path ]
then
    # NOTE(saga): Virtualenv binary not found. Try and use python3's virtualenv
    # module. However it doesn't support specifying custom python_exe path (-p)
    python3 -m venv /usr/share/%{name}-%{version}/invirtualenv_deployer
else
    $virtualenv_path -p {{global['basepython']}} /usr/share/%{name}-%{version}/invirtualenv_deployer
fi
/usr/share/%{name}-%{version}/invirtualenv_deployer/bin/pip install -q --no-index --find-links=/usr/share/%{name}-%{version}/wheels invirtualenv configparser
cd /usr/share/%{name}-%{version}
#/usr/share/%{name}-%{version}/invirtualenv_deployer/bin/deploy_virtualenv
/usr/share/%{name}-%{version}/invirtualenv_deployer/bin/python /usr/share/%{name}-%{version}/package_scripts/post_install.py

%preun
/usr/share/%{name}-%{version}/invirtualenv_deployer/bin/python /usr/share/%{name}-%{version}/package_scripts/pre_uninstall.py

%postun
rm -rf /usr/share/%{name}-%{version}

%files
%defattr(0755, root, root)
/usr/share/%{name}-%{version}/*
{% if rpm_package['files'] %}
{% endif %}
"""

RPM_CONFIG_DEFAULT = """[rpm_package]
deps:
"""


class InvirtualenvRPM(InvirtualenvPlugin):
    package_formats = ['rpm']
    package_template = SPEC_TEMPLATE
    config_default = RPM_CONFIG_DEFAULT
    config_types = {
        'rpm_package': {
            'deps': list
        }
    }
    default_config_filename = 'invirtualenv.spec'

    def system_requirements_ok(self):
        if find_executable('rpmbuild'):
            return True
        logger.debug('The rpmbuild command is not present, disabling the rpm plugin')
        return False

    def run_package_command(self, package_hashes, wheel_dir='wheels'):
        # Make sure the configuration is sane
        # self.config['rpm_package']['deps'].append('invirtualenv')
        self.config['rpm_package']['cwd'] = os.getcwd()
        description = self.config['global'].get('description', '').strip()
        if not description:
            self.config['global']['description'] = 'No description available'

        # Get the packaging script
        for script in ['rpm_scripts/post_install.py', 'rpm_scripts/pre_uninstall.py']:
            with open(os.path.basename(script), 'wb') as script_handle:
                script_handle.write(pkgutil.get_data('invirtualenv_plugins', script))

        logger.debug('Config')
        logger.debug(self.config)
        logger.debug('Spec')
        logger.debug(self.render_template_with_config())
        logger.debug('Deploy.conf')
        logger.debug(open('deploy.conf').read())
        with open('package.spec', 'w') as spec_handle:
            spec_handle.write(self.render_template_with_config())
        os.system('ls -lR')
        command = [find_executable('rpmbuild'), '-ba', 'package.spec']
        logger.debug('Running command %r', ' '.join(command))
        output = subprocess.check_output(command, env={'LANG': 'C'})
        output = output.decode(errors='ignore')
        packages = []
        for line in output.split('\n'):
            line = line.strip()
            if line.startswith('Wrote: '):
                packages.append(line.split()[-1])
        logger.debug('found packages %r', packages)
        if packages:
            return packages[-1]
